@@ -6,5 +6,6 @@ function(region) {
   	for (i in 2:length(region)) qry <- paste(qry,region[i],sep="+OR+");
   }
   url <- paste("http://www.ncbi.nlm.nih.gov/mapview/map_search.cgi?taxid=9606&query=",qry,"&qchr=&strain=All&advsrch=off",sep="");
-  browseURL(url);
+  a <- try(browseURL(url));
+  if (class(a) == "try-error") stop("can not invoke browser")
 }
